/*
 * wave.c
 *
 * Copyright(c) 2000, Particle
 */

#include <windows.h>
#include <scrnsave.h>
#include <math.h>

#include "resource.h"

typedef BYTE byte_t;

int W_WIDTH = 320;
int W_HEIGHT = 240;

struct pBITMAPINFO{
	BITMAPINFOHEADER bmiHeader;
	RGBQUAD bmiColors[256];
}BMInfo;

struct pLOGPALETTE{
	WORD	palVersion;
	WORD	palNumEntries;
	PALETTEENTRY palPalEntry[256];
}PalInfo;

HBITMAP hBM;
byte_t* double_buffer;

int* sin_x_table;
int* sin_y_table;
int xcounter = 0;
int ycounter = 0;
int mover = 40;

void p_update(){

	int m_width = W_WIDTH;
	int m_height = W_HEIGHT;

	int x,y,tx,ty;
	mover = mover > 1000000 ? 40:(mover+1);
	xcounter = xcounter > (m_width-10) ? 0:xcounter+9;
	ycounter = ycounter > (m_height-6) ? 0:ycounter+5;
	for(y=0;y<m_height;y++)
		for(x=0;x<m_width;x++){
			tx = x + sin_y_table[y + ycounter]+mover;
			ty = y - sin_x_table[x + xcounter]+mover;
			double_buffer[y*m_width+x] = (BYTE)(tx ^ ty);
		}
}

void getpalette(RGBQUAD* p){
	int i;
	for(i=0;i<256;i++){
		p[i].rgbRed = i;
		p[i].rgbGreen = i;
		p[i].rgbBlue = 100;
		p[i].rgbReserved = 0;
	}
}
		
void init_double_buffer(HDC hDC){
	HPALETTE PalHan;
	RGBQUAD palette[256];
	int i;


	BMInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	BMInfo.bmiHeader.biWidth = W_WIDTH;
	BMInfo.bmiHeader.biHeight = -abs(W_HEIGHT);
	BMInfo.bmiHeader.biPlanes = 1;
	BMInfo.bmiHeader.biBitCount = 8;
	BMInfo.bmiHeader.biCompression = BI_RGB;
	BMInfo.bmiHeader.biSizeImage = 0;
	BMInfo.bmiHeader.biXPelsPerMeter = 0;
	BMInfo.bmiHeader.biYPelsPerMeter = 0;
	BMInfo.bmiHeader.biClrUsed = 256;
	BMInfo.bmiHeader.biClrImportant = 256;

	getpalette(palette);

	for(i=0;i<256;i++)
		BMInfo.bmiColors[i] = palette[i];

	PalInfo.palVersion = 0x300;
	PalInfo.palNumEntries = 256;
	for(i=0;i<256;i++){
		PalInfo.palPalEntry[i].peRed = palette[i].rgbRed;
		PalInfo.palPalEntry[i].peGreen = palette[i].rgbGreen;
		PalInfo.palPalEntry[i].peBlue = palette[i].rgbBlue;
		PalInfo.palPalEntry[i].peFlags = PC_NOCOLLAPSE;
	}

		/* create the palette */
	PalHan = CreatePalette((LOGPALETTE*)&PalInfo);
		/* select it for that DC */
	SelectPalette(hDC,PalHan,FALSE);
		/* realize a palette on that DC */
	RealizePalette(hDC);
		/* delete palette handler */
	DeleteObject(PalHan);

	hBM = CreateDIBSection(hDC,(BITMAPINFO*)&BMInfo,
		DIB_RGB_COLORS,(void**)&double_buffer,0,0);

}

void blit(HDC hDC,RECT Dest){
	HDC Context;
	HBITMAP DefaultBitmap;	
	Context = CreateCompatibleDC(0);
	DefaultBitmap = SelectObject(Context,hBM);
	BitBlt(hDC,0,0,Dest.right,Dest.bottom,Context,0,0,SRCCOPY);
	SelectObject(Context,DefaultBitmap);
	DeleteDC(Context);
	DeleteObject(DefaultBitmap);
}












LRESULT WINAPI ScreenSaverProc(HWND hwnd,UINT message,WPARAM wParam,LPARAM lParam){
	static HDC hdc,memdc;
	static RECT rect;
	static unsigned int timer;
	static HBITMAP hBitmap;
	static BYTE* double_buffer;

	int m_width = W_WIDTH;
	int m_height = W_HEIGHT;

	int i;

	switch(message){
		case WM_CREATE:

			hdc = GetDC(hwnd);
			GetClientRect(hwnd,&rect);
				
			W_WIDTH = rect.right;
			W_HEIGHT = rect.bottom;
			
			m_width = W_WIDTH;
			m_height = W_HEIGHT;

			sin_x_table = (int*)malloc(m_width*2*sizeof(int));
			sin_y_table = (int*)malloc(m_height*2*sizeof(int));
			
			for(i=0;i<m_width;i++)
				sin_x_table[i] = (int)(sin(i * 6.28/m_width) * 50);
			for(;i<m_width*2;i++)
				sin_x_table[i] = sin_x_table[i-m_width];
			for(i=0;i<m_height*2;i++)
				sin_y_table[i] = (int)(sin(i * 6.28/m_height) * 50);
			for(;i<m_height;i++)
				sin_y_table[i] = sin_y_table[i-m_height];		

			init_double_buffer(hdc);
		
			ReleaseDC(hwnd,hdc);

			timer = SetTimer(hwnd,1,2,NULL);
			break;
		case WM_TIMER:
			p_update();
			hdc = GetDC(hwnd);
			GetClientRect(hwnd,&rect);
			blit(hdc,rect);
			ReleaseDC(hwnd,hdc);
			break;
		case WM_ERASEBKGND:
			break;
		case WM_DESTROY:
			KillTimer(hwnd,timer);
			break;
		default:
			return DefScreenSaverProc(hwnd,message,wParam,lParam);
	}
	return 0;
}


BOOL WINAPI ScreenSaverConfigureDialog(HWND hdwnd,UINT message,WPARAM wParam,LPARAM lParam){
	return 0;
	
}

BOOL WINAPI RegisterDialogClasses(HINSTANCE hInst){
	return 1;
}

